/***************************************************************************//**
*   @file   cn0202.c
*   @brief  Implementation of CN0202 Driver for NIOS processor.
*   @author DBogdan (dragos.bogdan@analog.com)
*
********************************************************************************
* Copyright 2012(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
********************************************************************************
*   SVN Revision: 706
*******************************************************************************/
#define CN0202_GLOBALS

/******************************************************************************/
/* Include Files															  */
/******************************************************************************/
#include "cn0202.h"
#include <system.h>
#include <altera_avalon_pio_regs.h>	// PIO access macros.
#include <altera_avalon_spi_regs.h> // SPI

#if (USE_CN0202 == 1)

/******************************************************************************/
/*  Private variables                                                         */
/******************************************************************************/
volatile unsigned char rangeConfig[22] = {0x05, // 0 V to 5 V.
										  0x09, // 0 V to 6.0 V (20% overrange).
										  0x06, // 0 V to 10 V.
										  0x0A, // 0 V to 12.0 V (20% overrange).
										  0x0D, // 2.5 V.
										  0x07, // 5 V.
										  0x0B, // 6.0 V (20% overrange).
										  0x08, // 10 V.
										  0x0C, // 12.0 V (20% overrange).
										  0x10, // 4 mA to 20 mA (internal current sense resistor).
										  0x00, // 4 mA to 20 mA (external 15 k current sense resistor).
										  0x11, // 0 mA to 20 mA (internal current sense resistor).
										  0x01, // 0 mA to 20 mA (external 15 k current sense resistor).
										  0x12, // 0 mA to 24 mA (internal current sense resistor).
										  0x02, // 0 mA to 24 mA (external 15 k current sense resistor).
										  0x13, // 20 mA (internal current sense resistor).
										  0x03, // 20 mA (external 15 k current sense resistor).
										  0x14, // 24 mA (internal current sense resistor).
										  0x04, // 24 mA (external 15 k current sense resistor).
										  0x1D, // 3.92 mA to 20.4 mA (internal current sense resistor).
										  0x1E, // 0 mA to 20.4 mA (internal current sense resistor).
										  0x1F};// 0 mA to 24.5 mA (internal current sense resistor).
volatile unsigned int serialWord		= 0x0;	// Serial word sent to AD5750.
volatile unsigned char faultCheck		= 0x0;	// Value of the Fault Pin.
volatile unsigned short statusRegister	= 0x0;	// Value of the Status Register.

/******************************************************************************/
/*  Aliases for the uC Probe data                                             */
/******************************************************************************/
volatile unsigned short*	ad5662WrValue	= &CN0202_data[0].usiData;	// AD5662 write value.
volatile unsigned char*		ad5662WrButton	= &CN0202_data[1].uchData;	// AD5662 write button.
volatile unsigned char*		ad5750Jp1		= &CN0202_data[2].uchData;	// AD5750 JP1 position.
volatile unsigned char*		ad5750RangeVal	= &CN0202_data[3].uchData;	// AD5750 range slider.
volatile unsigned char*		ad5750ModifyRg	= &CN0202_data[4].uchData;	// AD5750 modify range button.
volatile unsigned char*		ad5750PecError	= &CN0202_data[5].uchData;	// AD5750 interface error.
volatile unsigned char*		ad5750OverTemp	= &CN0202_data[6].uchData;	// AD5750 temperature error.
volatile unsigned char*		ad5750IoutFault	= &CN0202_data[7].uchData;	// AD5750 open circuit on the IOUT pin.
volatile unsigned char*		ad5750VoutFault	= &CN0202_data[8].uchData;	// AD5750 short circuit on the VOUT pin.
volatile unsigned char*		ad5750ClrButton	= &CN0202_data[9].uchData;	// AD5750 clear button.

/***************************************************************************//**
 * @brief Writes 24 bits to SPI.
 *
 * @param base - Base address.
 * @param syncPin - Sync pin of the device.
 * @param data - Data which will be transmitted..
 *
 * @return None.
*******************************************************************************/
void SPI_Write24Bits(unsigned int base,
					 unsigned char syncPin,
					 unsigned int data)
{
	IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(SYNC_SIGNALS_BASE, syncPin);

	while((IORD_ALTERA_AVALON_SPI_STATUS(base) &
		   ALTERA_AVALON_SPI_STATUS_TRDY_MSK) == 0);			 // Wait until txdata register is empty.
	IOWR_ALTERA_AVALON_SPI_TXDATA(base, (data & 0xFF0000) >> 16);// Put data in txdata register.

	while((IORD_ALTERA_AVALON_SPI_STATUS(base) &
		   ALTERA_AVALON_SPI_STATUS_TRDY_MSK) == 0);			// Wait until txdata register is empty.
	IOWR_ALTERA_AVALON_SPI_TXDATA(base, (data & 0x00FF00) >> 8);// Put data in txdata register.

	while((IORD_ALTERA_AVALON_SPI_STATUS(base) &
		   ALTERA_AVALON_SPI_STATUS_TRDY_MSK) == 0);			// Wait until txdata register is empty.
	IOWR_ALTERA_AVALON_SPI_TXDATA(base, (data & 0x0000FF) >> 0);// Put data in txdata register.

	while((IORD_ALTERA_AVALON_SPI_STATUS(base) &
		   ALTERA_AVALON_SPI_STATUS_TMT_MSK) == 0);				// Wait until the interface has finished transmitting.

	IOWR_ALTERA_AVALON_PIO_SET_BITS(SYNC_SIGNALS_BASE, syncPin);
}

/***************************************************************************//**
 * @brief Writes 16 bits to SPI.
 *
 * @param base - Base address.
 * @param syncPin - Sync pin of the device.
 * @param data - Data which will be transmitted..
 *
 * @return None.
*******************************************************************************/
void SPI_Write16Bits(unsigned int base,
					 unsigned char syncPin,
					 unsigned int data)
{
	IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(SYNC_SIGNALS_BASE, syncPin);

	while((IORD_ALTERA_AVALON_SPI_STATUS(base) &
		   ALTERA_AVALON_SPI_STATUS_TRDY_MSK) == 0);			// Wait until txdata register is empty.
	IOWR_ALTERA_AVALON_SPI_TXDATA(base, (data & 0xFF00) >> 8);	// Put data in txdata register.

	while((IORD_ALTERA_AVALON_SPI_STATUS(base) &
		   ALTERA_AVALON_SPI_STATUS_TRDY_MSK) == 0);			// Wait until txdata register is empty.
	IOWR_ALTERA_AVALON_SPI_TXDATA(base, (data & 0x00FF) >> 0);	// Put data in txdata register.

	while((IORD_ALTERA_AVALON_SPI_STATUS(base) &
		   ALTERA_AVALON_SPI_STATUS_TMT_MSK) == 0);				// Wait until the interface has finished transmitting.

	IOWR_ALTERA_AVALON_PIO_SET_BITS(SYNC_SIGNALS_BASE, syncPin);
}

/***************************************************************************//**
 * @brief Reads 16 bits from SPI.
 *
 * @param base - Base address.
 * @param syncPin - Sync pin of the device.
 * @param data - Data which will be transmitted (MOSI pin).
 *
 * @return receivedData - Received data (MISO pin).
*******************************************************************************/
unsigned short SPI_Read16Bits(unsigned int base,
							  unsigned char syncPin,
							  unsigned int data)
{
	unsigned short receivedData;

	IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(SYNC_SIGNALS_BASE, syncPin);

	IORD_ALTERA_AVALON_SPI_RXDATA(base);						// Discard any stale data, in case previous
																// communication was interrupted.
	while((IORD_ALTERA_AVALON_SPI_STATUS(base) &
		   ALTERA_AVALON_SPI_STATUS_TRDY_MSK) == 0);			// Wait until txdata register is empty.
	IOWR_ALTERA_AVALON_SPI_TXDATA(base, (data & 0xFF00) >> 8);	// Generate 8 clocks.
	while ((IORD_ALTERA_AVALON_SPI_STATUS(base) &
			ALTERA_AVALON_SPI_STATUS_RRDY_MSK) == 0);			// Wait until rxdata register is full.
	receivedData = IORD_ALTERA_AVALON_SPI_RXDATA(base);			// Read data from rxdata register.
	receivedData = receivedData << 8;

	while((IORD_ALTERA_AVALON_SPI_STATUS(base) &
		   ALTERA_AVALON_SPI_STATUS_TRDY_MSK) == 0);			// Wait until txdata register is empty.
	IOWR_ALTERA_AVALON_SPI_TXDATA(base, (data & 0x00FF) >> 0);	// Generate 8 clocks.
	while ((IORD_ALTERA_AVALON_SPI_STATUS(base) &
			ALTERA_AVALON_SPI_STATUS_RRDY_MSK) == 0);			// Wait until rxdata register is full.
	receivedData += IORD_ALTERA_AVALON_SPI_RXDATA(base);		// Read data from rxdata register.

	while((IORD_ALTERA_AVALON_SPI_STATUS(base) &
		   ALTERA_AVALON_SPI_STATUS_TMT_MSK) == 0);				// Wait until the interface has finished transmitting.

	IOWR_ALTERA_AVALON_PIO_SET_BITS(SYNC_SIGNALS_BASE, syncPin);

	return(receivedData);
}

/***************************************************************************//**
 * @brief Initializes SPI and initial values for CN0202 board.
 *
 * @param None.
 *
 * @return Always returns success.
*******************************************************************************/
int CN0202_Init(void)
{
	/* Deselect AD5750 and AD5662. */
	IOWR_ALTERA_AVALON_PIO_SET_BITS(SYNC_SIGNALS_BASE, AD5750_SYNC_PIN |
													   AD5662_SYNC_PIN);
	SetCtrl(CN0202_SPI_CFG);	// Setup SPI interface.
	/* AD5750_CLR_PIN - output; AD5750_FAULT_PIN - input. */
	IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, AD5750_CLR_PIN);
	IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, AD5750_CLR_PIN);

	return(0);
}

/**************************************************************************//**
 * @brief Updates the device's monitored and controlled parameters.
 *
 * @param None.
 *
 * @return Always returns success.
******************************************************************************/
int CN0202_UpdateParameters(void)
{
	faultCheck = IORD_ALTERA_AVALON_PIO_DATA(GPIO_BASE);
	faultCheck = (faultCheck & AD5750_FAULT_PIN);
	if(faultCheck == 0)	// If the Fault Alert Pin is active
	{					// reads the Status Register.
		serialWord = AD5750_STATUS_ADDR(*ad5750Jp1) +
					 AD5750_STATUS_READ;
		statusRegister = SPI_Read16Bits(CN0202_SPI,
										AD5750_SYNC_PIN,
										serialWord);
		*ad5750PecError = (statusRegister & AD5750_STATUS_PEC_ERR) >> 3;
		*ad5750OverTemp = (statusRegister & AD5750_STATUS_OVER_TEMP) >> 2;
		*ad5750IoutFault = (statusRegister & AD5750_STATUS_IOUT_FAULT) >> 1;
		*ad5750VoutFault = (statusRegister & AD5750_STATUS_VOUT_FAULT) >> 0;
	}
	if(*ad5750ClrButton)	// If the AD5750 Clear Pin button was pressed
	{						// toggles the CLEAR pin.
		IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, AD5750_CLR_PIN);
	}
	else
	{
		IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, AD5750_CLR_PIN);
	}
	if(*ad5750ModifyRg)	// If the Modify Range button was pressed
	{					// modifies the AD5750 output range.
		*ad5750PecError = 0;
		*ad5750OverTemp = 0;
		*ad5750IoutFault = 0;
		*ad5750VoutFault = 0;
		serialWord = AD5750_CONTROL_ADDR(*ad5750Jp1) +
					 AD5750_CONTROL_WRITE +
					 AD5750_CONTROL_RANGE(rangeConfig[*ad5750RangeVal] & 0xF) +
					 AD5750_CONTROL_OUTEN +
					 ((rangeConfig[*ad5750RangeVal] & 0x10) >> 1);
		SPI_Write16Bits(CN0202_SPI, AD5750_SYNC_PIN, serialWord);
		*ad5750ModifyRg = 0;
	}
	if(*ad5662WrButton)	// If the Write AD5662 button was pressed
	{					// loads the *ad5662WrValue value to AD5662.
		SPI_Write24Bits(CN0202_SPI,
						AD5662_SYNC_PIN,
						AD5662_LOAD + *ad5662WrValue);
		*ad5662WrButton = 0;
	}

	return (0);
}
#endif /* USE_CN0202 */
